/*
 *   This file is part of Clinica.
 *
 *   Clinica is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Clinica is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Clinica.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Authors: Leonardo Robol <leo@robol.it>
 *            Gianmarco Brocchi <brocchi@poisson.phc.unipi.it>
 */

using Gtk;

namespace Clinica {

    public class PatientEditor : GLib.Object  {
    
    	public enum Response {
    	    SAVE = 0,
    	    CANCEL = 1,
    	}
    	
    	public enum DoctorCompletionAction {
    		CREATE_NEW = 0,
    	}
    
        public signal void error (string message);
        private Builder builder;

#if HAVE_GTK3
        private ComboBoxText gender_combobox;
#else
		private ComboBox gender_combobox;
#endif
        
        /* Gtk entries from where data is taken when creating
         * the new patient */
        private Entry given_name_entry;
        private Entry surname_entry;
        private Entry phone_entry;
        private Entry residence_address_entry;
        private Entry codice_fiscale_entry;
        private Entry day_entry;
        private Entry month_entry;
        private Entry year_entry;
        private Entry doctor_entry;
        
        /* Selected doctor is set from matched signal
         * of the autocompletion. */
        private Doctor? selected_doctor = null;
        
        // Set to a patient that is the existing object for the
        // one represented here. 
        private Patient? existing_patient = null;
        
        /* Dialog opened to select data of the new patient */
        private Dialog dialog;

		private Button save_button;
		
		/* Completion for doctors */
		private EntryCompletion completion;
    	
        public PatientEditor () {
        	error.connect ((t,l) => Application.resources.error_callback (t,l));
            
            builder = new Builder ();
            try {
                builder.add_from_file (Application.resources.get_ui_file ("patient_editor.glade"));
            } catch (Error e) {
                error (_("Error loading patient_editor.glade."));
            }
            
            /* Fill the dialog with the pieces from the gtk builder file */ 
            dialog = builder.get_object("patient_editor") as Dialog;
            
            /* Load entries into local object */
            doctor_entry = builder.get_object ("doctor_entry") as Entry;
            given_name_entry = builder.get_object ("given_name_entry") as Entry;
            surname_entry = builder.get_object ("surname_entry") as Entry;
            phone_entry = builder.get_object ("phone_entry") as Entry;
            residence_address_entry = builder.get_object ("residence_address_entry") as Entry;
            codice_fiscale_entry = builder.get_object ("codice_fiscale_entry") as Entry;
            day_entry = builder.get_object ("day_entry") as Entry;
            month_entry = builder.get_object ("month_entry") as Entry;
            year_entry = builder.get_object ("year_entry") as Entry;
            
#if HAVE_GTK3
            gender_combobox = builder.get_object ("gender_combobox") as ComboBoxText;
#else
            gender_combobox = builder.get_object ("gender_combobox") as ComboBox;
#endif

	    	/* Make save_button insensitive until user does not select a valid doctor. */
            save_button = builder.get_object ("save_button") as Button;
	    	save_button.set_sensitive (false);
            
            /* Create Doctor Entry with autocompletion, and setup a callback
             * to store the Doctor selected with autocompletion when this 
             * happens. */
            completion = new EntryCompletion ();
            completion.set_model (Application.resources.doctor_list_store);
            completion.set_text_column (3);
            completion.set_match_func (completion_match_function);
            completion.insert_action_text (DoctorCompletionAction.CREATE_NEW, "Create a new doctor");
            
            /* Callback that stores the selected doctor */
            completion.match_selected.connect (on_match_selected);
            
            /* Callback for action selected */
            completion.action_activated.connect (on_action_activated);
            
            /* Activate completion */
            doctor_entry.set_completion (completion);
            doctor_entry.changed.connect (on_doctor_entry_changed);
            
            /* Connect date change */
            day_entry.changed.connect (on_date_entry_changed);
            month_entry.changed.connect (on_date_entry_changed);
            year_entry.changed.connect (on_date_entry_changed);
            
            builder.connect_signals (this);
        }
        
        private void on_doctor_entry_changed (Editable editable) {
        	Entry entry = (editable as Entry);
	        string doc_name = entry.get_text ();
	    	
	    	/* Replace action */
	    	completion.delete_action (DoctorCompletionAction.CREATE_NEW);
	    	if (doc_name != "")
	        	completion.insert_action_text (DoctorCompletionAction.CREATE_NEW,
	        		@"Create a new doctor named $(Utils.capitalize (doc_name))");
	        else 
	        	completion.insert_action_text (DoctorCompletionAction.CREATE_NEW,
	        		@"Create a new doctor");

			/* If this is a valid doctor make "save" clikable, otherwise set it to inactive */
			check_input_data ();
        }        
        
        private void on_date_entry_changed (Editable editable) {
			check_input_data ();
		}

        /**
         * @brief Check if patient age and doctor have been input
         * correctly by the user and set the sensitivity of the
         * save_button according to this. 
         */
		private void check_input_data () {
			bool is_data_consistent = true;
        	        		
	        /* Check DateTime */
            int year = int.parse (year_entry.get_text ());
            int month = int.parse (month_entry.get_text ());
            int day = int.parse (day_entry.get_text ());
    		
    		var now = new DateTime.now_local ();
 		    int current_year = now.get_year ();
    		
    		/* If this is a valid date make "save" clikable, otherwise set it to inactive */      	
            if ((current_year - year > 150) || 
            	(month < 1) || (month > 12) ||
            	(day < 1)   || (day   > 31) ||
				!Utils.check_date (day, month, year)) 
            {
                is_data_consistent = false;
            }

			/* Check for the doctor validity */
			if (selected_doctor == null ||
				selected_doctor.get_complete_name () != doctor_entry.get_text ()) 
		    {
				is_data_consistent = false;
			}
			
			if (is_data_consistent)
				save_button.set_sensitive (true);
			else
				save_button.set_sensitive (false);
        }
        
        /*
         * @brief NewPatientDialog with an existing patient, to edit it
         */
        public PatientEditor.with_patient (Patient p) {
        	this ();
        	
        	// Load doctors and existing patient
        	selected_doctor = p.doctor;
        	existing_patient = p;
        	doctor_entry.set_text (p.doctor.get_complete_name ());
        	
        	// Pre fill the fields in the interface
        	given_name_entry.set_text (p.given_name);
        	surname_entry.set_text (p.surname);
        	phone_entry.set_text (p.phone);
        	residence_address_entry.set_text (p.residence_address);
        	codice_fiscale_entry.set_text (p.identification_code);
        	
        	// Parse gender into combobox
        	if (p.gender == Gender.MALE)
        		gender_combobox.set_active (0);
        	else
        		gender_combobox.set_active (1);
        		
        	// Parse date
        	var birth_date = p.birth_date;
        	day_entry.set_text (birth_date.get_day_of_month ().to_string ());
        	month_entry.set_text (birth_date.get_month ().to_string ());
        	year_entry.set_text (birth_date.get_year ().to_string ());
        	
        	// Set title to edit patient * instead of create new patient
        	dialog.set_title (_("Edit patient named %s").printf (p.get_complete_name ()));
        }
        
        /**
         * @brief Capitalize Patient's name and surname
         */
        [CCode (instance_pos = -1)]
        public bool on_name_focus_out_event (Gtk.Widget source, Gdk.Event event) {
        	string name = given_name_entry.get_text ();    	
        	if (name != "")
        		given_name_entry.set_text ( Utils.capitalize (name) );
        	return false;
        }
        
        [CCode (instance_pos = -1)]
        public bool on_surname_focus_out_event (Gtk.Widget source, Gdk.Event event) {
        	string surname = surname_entry.get_text ();    	
        	if (surname != "")
        		surname_entry.set_text ( Utils.capitalize (surname) );
        	return false;
        }

        /**
         * @brief Open a NewPatientDialog precompiled with the name
         * of the patient.
         */
        public PatientEditor.with_name (string name) {
        	this ();
        	string [] name_pieces = name.split (" ");
        	if (name_pieces.length > 0) {
        		given_name_entry.set_text (name_pieces[0]);
        	}
        	if (name_pieces.length > 1) {
        		surname_entry.set_text (string.joinv(" ", name_pieces[1:name_pieces.length]));
        	}
        }
        
        /**
         * @brief Create a new doctor on the fly with the name specifed in the 
         * doctor_entry
         */
        private void on_action_activated (EntryCompletion editable, int action) {
        	if (action != DoctorCompletionAction.CREATE_NEW)
            	return;
            	
            /* Create a new dialog precompiled with the user input until now */
            string doc_name = doctor_entry.get_text ();
            var dialog = new DoctorEditor.with_name (Utils.capitalize (doc_name));
            if (dialog.run () == DoctorEditor.Response.CANCEL) {
            	dialog.destroy ();
            	return;
            }
            
            /* Set the newly created doctor as selected doctor */
            selected_doctor = dialog.created_doctor;
            doctor_entry.set_text (selected_doctor.get_complete_name ());
            
            /* Destroy the dialog */
            dialog.destroy ();
            
            /* Check if all the data is now correct */
            check_input_data ();
        }
        
        /**
         * @brief Callback function for the doctor selection that matches the doctor
         * if the key is a substring of its complete name. 
         */
        private bool completion_match_function (EntryCompletion compl, string key, TreeIter iter) {
        	Value value;
        	Application.resources.doctor_list_store.get_value(iter, 0, out value);
        	Doctor doc = value as Doctor;
        	if (doc == null)
        		return false;
        	return (key.up () in doc.get_complete_name ().up ());
        }
        
        private bool on_match_selected (EntryCompletion compl, TreeModel model, TreeIter iter) {
        	/* Get doctor associated with the selected completion */
        	Value value;
        	model.get_value (iter, 3, out value);
        	doctor_entry.set_text (value as string);
        	model.get_value (iter, 0, out value);
        	
        	/* Set selected doctor in this object to use it later */
        	selected_doctor = value as Doctor;
        	
        	/* Check if all the input data is now correct and make save_button
        	 * sensitive according to this */
        	check_input_data ();
        	return true;
        }
        
        public Response run () {
            if (dialog.run () == 0) {
            /* Check if there exists a selected doctor and if that
         	 * doctor is still the selected one. */
            	if (selected_doctor == null || doctor_entry.get_text() != selected_doctor.get_complete_name ()) {
            		
		       		error (_("You must select a doctor for this patient.") + 
			        	_("If you need to create a new one type his name in the entry and select Create new doctor from the completion list."));
		        	return Response.CANCEL;
		        }
            	
            	/*  Save the new patient, creating a new one if this is a new patient, or
            	 *  reusing the existing one if we are only modifying a patient. */
            	Patient patient;
            	if (existing_patient == null)
	            	patient = new Patient (selected_doctor);
	            else
					patient = existing_patient;
					
            	patient.given_name = given_name_entry.get_text ();
            	patient.surname = surname_entry.get_text ();
            	patient.phone = phone_entry.get_text ();
            	patient.identification_code = codice_fiscale_entry.get_text ();
            	patient.residence_address = residence_address_entry.get_text ();
            	
            	/* Create new DateTime */
            	int year = int.parse (year_entry.get_text ());
            	int month = int.parse (month_entry.get_text ());
            	int day = int.parse (day_entry.get_text ());
    			DateTime t;
    					
            	if ((1 <= year) &&  (year <= 9999) && 
            		(1 <= month) && (month <= 12)  &&
            		(1 <= day) && (day <= 31))
            	{
	            	t = new DateTime (new TimeZone.utc (), year, month, day, 0, 0, 0);
	            	patient.birth_date = t;
	            }
	            else
	            {
//	            	if ((day_entry.get_text() == "") && (month_entry.get_text() == "") && (year_entry.get_text() == ""))
//	            	if (year == 0);
//	            		patient.birth_date = null;
	            		
//	            	else {
//	            		print ("year: %d, month: %d, day: %d", year, month, day);
	            		save_button.set_sensitive (false);
	            		error (_("Date inserted is invalid, aborting patient editing"));
	            		return Response.CANCEL;
//	            	}	            	
	            }
	            
//            	patient.birth_date = t;
            	
            	/* Set gender here */
            	if (gender_combobox.get_active () == 0)
	            	patient.gender = Gender.MALE;
	            else
	            	patient.gender = Gender.FEMALE;
	            
            	patient.save ();
            	
            	// If this was an existing patient we need to reload it in the patient
            	// list store, otherwise we can just add it
            	if (existing_patient != null) {
            		Application.resources.patient_list_store.reload_patient (patient);
            	} else {
	            	Application.resources.patient_list_store.add_patient (patient);
	            }
            	
            	
                return Response.SAVE;
            }
            
            return Response.CANCEL;
        }
        
        public void destroy () {
            dialog.destroy ();
        }
    }
}

