/*
 *   This file is part of Clinica.
 *
 *   Clinica is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Clinica is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Clinica.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Authors: Leonardo Robol <leo@robol.it>
 *            Gianmarco Brocchi <brocchi@poisson.phc.unipi.it>
 */

using GLib;
using Sqlite;

namespace Clinica {

    /**
     * @brief Grant access to resources such as
     * UI files, configuration and database
     */
    public class ResourceManager : GLib.Object {
    
        /**
         * @brief Callback that manages errors
         * notification to the end user.
         */
        public unowned ErrorCallback error_callback;
        
        /**
         * @brief PatientListStore that holds the list
         * of the patients
         */
        public PatientListStore patient_list_store;
        
        /**
         * @brief DoctorListStore that holds the list
         * of the doctors.
         */
        public DoctorListStore doctor_list_store;
        
        /**
         * @brief VisitListStore that holds the list
         * of the visits.
         */
        public VisitListStore visit_list_store;
    
        /**
         * @brief Signal emitted when an error
         * occurr. The user should be notified
         * with message by the application
         */
        public signal void error (string message);
    
        /**
         * @brief Local configuration directory
         * obtained at runtime.
         */
        private string local_config_directory;
        
        /**
         * @brief Database path.
         */
        private string database_path;
        
        /**
         * @brief SQLite database with data and
         * configurations.
         */
        public Database db;
        
        /**
         * @brief Path to the UI files directory
         */
        private string ui_files_path;
        private string icons_files_path;
        private string prefix;
        private string data_path;
    
        public ResourceManager (string program_name, ErrorCallback callback) {
        
            this.error_callback = callback;
        
            /* Connect error callback */
            error.connect ((t,l) => callback(t,l));
            
            /* Set up databases names */
            Doctor.table  = "doctors";
            Patient.table = "patients";
            Visit.table   = "visits";
        
            /* Init databases and configuration files
             * if needed. */
            local_config_directory = Environment.get_user_config_dir ();
            local_config_directory = Path.build_filename (local_config_directory,
                                                          "clinica");
            database_path = Path.build_filename (local_config_directory,
                                                 "clinica.db");
                                                 
            /* Check if configuration directory exists */
            if (!FileUtils.test (local_config_directory, 
                                  FileTest.IS_DIR)) {
                DirUtils.create_with_parents (local_config_directory, -1);
            }
                                                 
            /* Read database version if existant, saved actual version otherwise.
             * This may be used in future to perform one-time upgrade of the database */
            var version_db_file = Path.build_filename (local_config_directory, 
            										    ".clinica_db_version");
            										    
        	if (!FileUtils.test (version_db_file, FileTest.IS_REGULAR)) {
        		/* Create the file with the version of the database */
        		try {
	        		FileUtils.set_contents(version_db_file, "0.1");
	        	} catch (Error e) {
	        		error (_("Error creating some configuration files, check permission on %s".printf(version_db_file)));
	        	}
        	}
        	else {
        		string content;
        		try {
        			FileUtils.get_contents(version_db_file, out content);
        		} catch (Error e) {
        			error (_("Error reading some configuration files, check permission on %s".printf(version_db_file)));
        		}
        		if ("0.1" != content) {
        			error (_("Version of the database is not compatible"));
        		}
        	}
                                          
                                                 
            /* Find prefix */
            prefix = "/usr/local";
            if (!FileUtils.test (Path.build_filename(prefix, "bin", "clinica"), FileTest.IS_EXECUTABLE)) {
            	prefix = "/usr";
            }
            
            /* Check if this is a local build and if that's the case then
             * load local interface files. */
            bool is_local = true;
            string [] source_files = { "AboutDialog.vala", "Page.vala", "StartPage.vala" };
            string src_base = Path.get_dirname (program_name);
            foreach(string source in source_files) {
	            if (!FileUtils.test (Path.build_filename(src_base, source), FileTest.IS_REGULAR)) {
		            is_local = false;
	            }
            }
             
            if (is_local) {
            	data_path = Path.build_filename(Path.get_dirname(program_name), "..");
            } else {
	            data_path = Path.build_filename(prefix, "share", "clinica");
        	}
	       
            /* Find UI files */
            ui_files_path = Path.build_filename(data_path, "ui");
            
            /* Find icons */
            icons_files_path = Path.build_filename (ui_files_path, "icons");
            
            
        }
        
        public string get_ui_file (string filename) {
            return Path.build_filename (ui_files_path, filename);
        }
            
        public string get_image_file (string image_name) {
        	return Path.build_filename (icons_files_path, image_name);
        }
        
        /**
         * @brief Check that all the resources needed by Clinica
         * are actually available, and if they are not create them.
         */
        public void initResources () {
            int rc;
            Statement stmt;
            string sql;
            
            /* Open database and, if it does not exists, create it */
            if (!(Database.open (database_path, out db) == OK)) {
                error ("Error opening database.");
            };
            
            /* Check if the required tables exists */
            sql = "SELECT * from sqlite_master WHERE type='table';";
            db.prepare (sql, -1, out stmt, null);
            rc = stmt.step ();
            
            if (rc == DONE) {
                /* That means no tables in the database */
                initDatabase ();
            }

            /* Load stores */
            doctor_list_store = new DoctorListStore ();
            patient_list_store = new PatientListStore ();
            visit_list_store = new VisitListStore ();
        }
        
        
        /**
         * @brief Init the database with the required tables
         */
        private void initDatabase () {
            Statement stmt; 
            string sql;
            string err_msg;
            
            /* Create config table */
            sql = "CREATE TABLE config (id INTEGER PRIMARY KEY, " +
                  "key TEXT, value TEXT);";
            db.prepare (sql, -1, out stmt, null);
            if (!(stmt.step () == DONE)) {
                err_msg = db.errmsg ();
                error ("Error creating config structure in the Database: "
                + err_msg);
            }
        }            
        
        /**
         * @brief Return a Doctor object associated to the given ID
         */
        public Doctor getDoctor (int ID) {
            Doctor doctor = new Doctor.with_id (ID);
            return doctor;
        }
        
        /**
         * @brief Return a Patient object associated to the given ID
         */
        public Patient getPatient (int ID) {
            Patient p = new Patient.with_id (ID);
            return p;
        }
        
        /**
         * @brief Return a Visit object associated to the given ID
         */
        public Visit getVisit (int ID) {
            Visit visit = new Visit.with_id (ID);
            return visit;
        }
        
        
    }
   

}
