/*
 *   This file is part of Clinica.
 *
 *   Clinica is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Clinica is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Clinica.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Authors: Leonardo Robol <leo@robol.it>
 *            Gianmarco Brocchi <brocchi@poisson.phc.unipi.it>
 */

using Gtk;

namespace Clinica {

	/**
	 * @brief A toolbar for the VisitWindow window
	 */
	public class VisitToolbar : HBox {
	
		public enum ViewMode {
			VERBOSE,
			SHORT,
		}
	
		/* References to useful objects */
		private Patient patient;
		
		private Label patient_label;
		private ViewMode view_mode;
		
		/* Action area */
		private VisitActions visit_actions;
		
		/* Signals */
		public signal void save_visit ();
	
		public VisitToolbar (Patient patient) {
			/* Set default properties */
			GLib.Object (homogeneous: false, spacing: 6);
			this.patient = patient;
			
			/* Get image of the patient */
			var patient_image = new Gtk.Image.from_file (Application.resources.get_image_file ("patient.svg"));
			append (patient_image);
						
			/* A label to identify the patient, aligned to the left */
			patient_label = new Label (null);
			patient_label.set_alignment (0, 0.5F);
			pack_start (patient_label, true, true, 0);
			
			/* Create a new visit action and connect the button for saving
			 * the visit and the one for changing the view mode */
			visit_actions = new VisitActions ();
			visit_actions.save_visit.connect ((va) => save_visit ());
			visit_actions.show_details.connect ((va) => on_show_details_button_clicked ());
			append (visit_actions);
			
			show_patient_details (ViewMode.SHORT);
		}
		
		private void on_show_details_button_clicked () {
			if (view_mode == ViewMode.SHORT)
				show_patient_details (ViewMode.VERBOSE);
			else
				show_patient_details (ViewMode.SHORT);
		}
		
		/**
		 * @brief Show patient details according to the mode passed, 
		 * that can be ViewMode.SHORT or ViewMode.VERBOSE.
		 */
		private void show_patient_details (ViewMode mode) {		
			visit_actions.set_details_button_mode (mode);
			view_mode = mode;
			if (mode == ViewMode.SHORT) {
				patient_label.set_markup (_("Patient: <b>%s</b>\nDoctor: <b>%s</b>".printf(patient.get_complete_name (),
					patient.doctor.get_complete_name ())));
				
			} else {
				// Here the code does not show more details about this patient, but it should.
				// Since it is unclear what should be shown exactly, we keep a conservative
				// behaviour for now. 
				patient_label.set_markup (_("Patient: <b>%s</b>\nDoctor: <b>%s</b>".printf(patient.get_complete_name (),
					patient.doctor.get_complete_name ())));
			}
		}
		
		/**
		 * @brief Append a widget to the toolbar
		 */
		private void append (Widget widget) {
			pack_start (widget, false, false, 6);
		}
	}
	
}
